/*

2010-05-28 v0.2 te@tafeu.de
 - added cmdline options
2009-01-28 v0.1 te@tafeu.de

Adobe Flash:
HTTP policy files may no longer be used to authorize socket connections. 
Prior to version 9,0,115,0, an HTTP policy file, served from the master 
location of /crossdomain.xml on port 80, could be used to authorize a 
socket connection to any port 1024 or greater on the same host.

*/

#include <stdio.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <unistd.h>
#include <errno.h>
#include <signal.h>
#include <stdlib.h>

#define VERSION "0.2"
#define LISTENPORT 843
#define LISTENBACKLOG 7
static char* XMLTEMPLATE = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n    <cross-domain-policy xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xsi:noNamespaceSchemaLocation=\"http://www.adobe.com/xml/schemas/PolicyFileSocket.xsd\">\n        <allow-access-from domain=\"*\" to-ports=\"%s\" secure=\"false\" />\n        <site-control permitted-cross-domain-policies=\"master-only\" />\n    </cross-domain-policy>\n";

void usage(void);

int main (int argc, char **argv) {

  int sockfd = -1;
  int client = -1;
  int childpid = 0;
  struct sockaddr_in addr;
  socklen_t clientsize;
  char *portsval = "*";
  char XMLMSG[512] = "";

  int c, i;
  opterr = 1;
  while ((c = getopt(argc, argv, "Vvhp:")) != -1) {
    switch (c) {
      case 'p':
	for ( i = 0; optarg[ i ]; i++) { /* whee back to the core */
	  if ( optarg[i] != 44 && optarg[i] != 42 && (optarg[i] < 48 || optarg[i] > 57) ) {
	    fprintf(stderr, "\nERROR: argument '%s' to option 'p' is invalid!\n", optarg);
	    usage();
	    exit(1);
	  }
	}
	if (i > 30) { perror("\nERROR: argument to option 'p' is too big .."); exit(1); }
	portsval = optarg;
        break;
      case 'v':
      case 'V':
	printf("swf_socketauth version %s by Thomas Eckhardt (tafe.unkelhaeusser.net)\n", VERSION);
	exit(1);
      case 'h':
      case '?':
      default:
	usage();
	exit(1);
    }
  }

  if ( sprintf(XMLMSG, XMLTEMPLATE, portsval) < 0) {
	perror("failed to prepare XML");
	exit(1);
  }

  sockfd = socket (PF_INET, SOCK_STREAM, IPPROTO_TCP);
  if (sockfd != -1) {
    addr.sin_family = PF_INET;
    addr.sin_addr.s_addr = INADDR_ANY;
    addr.sin_port = htons(LISTENPORT);

    if ( bind(sockfd, (struct sockaddr *) &addr, sizeof(addr) ) != -1 ) {
      if( listen(sockfd, LISTENBACKLOG) != -1) {
	/* drop priv & fork */
        signal(SIGHUP, SIG_IGN);
	if (setgid( 1 ) == -1) { perror("can't setgid(1)"); }
        if (setuid( 1 ) == -1) { perror("can't setuid(1)"); }
        if ((childpid = fork()) < 0) { perror("can't fork");
        } else {
	  if (childpid > 0) { exit(0); } /* parent */
        }

        clientsize = sizeof(addr);
        for (;;) {
	   if ( (client = accept(sockfd, (struct sockaddr *)&addr, &clientsize)) != -1 ) {
	     if ( send(client, XMLMSG, sizeof(XMLMSG), MSG_DONTWAIT) == -1 ) { perror("send failed"); }
	     close(client);
	   } else { perror("error accepting connection"); }
        } /* endless loop end ;) */
      } else { perror("could not listen to socket"); }
    } else { perror("could not bind to socket"); }

    /* we will never get there, but to be complete .. :> */
    close (sockfd);

  } else { perror("could not open socket"); }

  return 0;
}

void usage(void)
{
  printf(
	"swf_socketauth by Thomas Eckhardt (tafe.unkelhaeusser.net)\n"
	"usage:\n"
	"  swf_socketauth [-p port(s)] || -h|--help\n\n"
	"  -p port(s)   port *to* which it's allowed to connect to, multiple\n"
	"               ports are seperated by commas, eg: -p 2001,4002 or -p 4000\n"
	"               If omitted, the default is to allow to connect to all ports,\n"
	"               this is the same as -p *\n"
	"  -h | --help  this help\n"
	);
  exit(1);
}
